"""Skilgen package."""

from skilgen.agents import fingerprint_project
from skilgen.autoupdate import auto_update_status, ensure_auto_update_worker, stop_auto_update_worker
from skilgen.delivery import run_delivery
from skilgen.sdk import (
    activate_project_mcp_connector,
    activate_skill_source,
    analyze_project,
    cancel_job,
    deactivate_project_mcp_connector,
    deactivate_skill_source,
    detect_skill_sources,
    deliver_project,
    generate_enterprise_skill_source,
    get_auto_update_status,
    get_job_status,
    ingest_enterprise_skill_source,
    init_project,
    list_active_mcp_connectors,
    install_skill_source,
    list_active_skill_sources,
    list_enterprise_skill_sources,
    list_mcp_connectors,
    list_project_jobs,
    list_skill_sources,
    map_codebase,
    plan_project,
    preview_project,
    project_report,
    project_status,
    rank_skill_sources,
    recommend_project_mcp_connectors,
    remove_skill_source,
    resume_job,
    show_skill_source,
    skill_source_lock,
    skill_source_policy,
    start_auto_update,
    start_deliver_job,
    stop_auto_update,
    sync_all_skill_sources,
    sync_skill_source,
    update_project,
    validate_project_outputs,
    watch_project,
)

__version__ = "0.4.0"

__all__ = [
    "__version__",
    "activate_project_mcp_connector",
    "activate_skill_source",
    "analyze_project",
    "auto_update_status",
    "cancel_job",
    "deactivate_project_mcp_connector",
    "deactivate_skill_source",
    "detect_skill_sources",
    "deliver_project",
    "fingerprint_project",
    "generate_enterprise_skill_source",
    "get_auto_update_status",
    "get_job_status",
    "ingest_enterprise_skill_source",
    "init_project",
    "list_active_mcp_connectors",
    "install_skill_source",
    "list_active_skill_sources",
    "list_enterprise_skill_sources",
    "list_mcp_connectors",
    "list_project_jobs",
    "list_skill_sources",
    "map_codebase",
    "plan_project",
    "preview_project",
    "project_report",
    "project_status",
    "rank_skill_sources",
    "recommend_project_mcp_connectors",
    "remove_skill_source",
    "resume_job",
    "run_delivery",
    "ensure_auto_update_worker",
    "show_skill_source",
    "skill_source_lock",
    "skill_source_policy",
    "start_auto_update",
    "start_deliver_job",
    "stop_auto_update",
    "stop_auto_update_worker",
    "sync_all_skill_sources",
    "sync_skill_source",
    "update_project",
    "validate_project_outputs",
    "watch_project",
]
